/* 
 * ===============================================================================
 * opensta2txt.cpp
 * -------------------------------------------------------------------------------
 *
 * Converts a binary file result produced by OpenSTA into a human-readable
 * text file
 *
 * Usage:	openstat2txt statname txtname
 *
 * -------------------------------------------------------------------------------
 * Author: Jerome Delamarche (c) inode-fr.com
 * Date  : 24 Aug 2006
 *
 * Added a flag to add the column names (suggested by Stian Thorvaldsen)
 * -------------------------------------------------------------------------------
 * Author: Jerome Delamarche (c) inode-fr.com
 * Date  : 14 Aug 2003
 *
 * ===============================================================================
 */
#include "stdafx.h"

#define VERSION "1.1"

#pragma pack(2)

/*
 * Record structure of OpenSTA binary file:
 */
typedef struct {
    DWORD		dwTimeStamp;	// timestamp of measure
	BYTE		bip1;			// four bytes of client address
	BYTE		bip2;
	BYTE		bip3;
	BYTE		bip4;
	DWORD		dwPID;
	WORD		wUser;
    DWORD		dwResponseTime;	// HTTP Response time in ms
    WORD		wResponseCode;	// HTTP Response code
    DWORD		dwReplySize;	// HTTP Reply Size in bytes
} OpenSTARecord;

/*
 * -----------------------------------------------------------------
 * Start the job !
 * ------------------------------------------------------------------
 */
int main(int argc, char *argv[])
{
	BOOL bHeader = FALSE;
	int  wFirstArg = 1;

	// Check option:
	if (argc > 1 && !strcmp(argv[1],"-h")) {
		bHeader = TRUE;
		wFirstArg = 2;
		argc--;
	}

	// 1st arg must be the "stat" filename
	// 2nd arg must be the resulting "txt" filename
	if (argc != 3) {
		fprintf(stderr,"Syntax: opensta2txt [-h] file.stat file.txt\n");
		fprintf(stderr,"Version: %s\n",VERSION);
		fprintf(stderr,"-h : add a line with the column name in file.txt\n");
		fprintf(stderr,"file.url must also exist (.stat and .url files are generated by OpenSTA)\n");
		exit(1);
	}

	/*
	 * Check the stat file and its associated "URL" file also:
	 */
	CString statFilename(argv[wFirstArg]);
	CFile *statCFile;
	try {
		statCFile = new CFile(statFilename,CFile::modeRead);
	}
	catch (CFileException e) {
		fprintf(stderr,"Binary OpenSTA File %s could not be opened in read mode: %s\n",
				statFilename,e.m_cause);
		return 0;
	}

	CString urlFilename(statFilename);
	urlFilename.Replace(".stat",".urls");
	CFile *urlFile;
	try {
		urlFile = new CFile(urlFilename,CFile::modeRead);
	}
	catch (CFileException e) {
		fprintf(stderr,"URL OpenSTA File %s could not be opened in read mode: %s\n",
				urlFilename,e.m_cause);
		return 0;
	}

	/*
	 * Open the output file:
	 */
	FILE *out = fopen(argv[wFirstArg+1],"w");
	if (!out) {
		fprintf(stderr,"Could not open file %s for writing\n",argv[wFirstArg+1]);
		return 0;
	}

	/*
	 * Add the column header if required:
	 */
	if (bHeader) {
		fprintf(out,"Timestamp,IP-PID-VU,Response Time,Response Code,Reply Size,URL\n");
	}

	/*
	 * Read the binary record one per one:
	 */
	OpenSTARecord record;
	char		  buff[2048];
	while (statCFile->Read(&record,sizeof(record)) == sizeof(record)) {
		// Dump each record in the output file:
		fprintf(out,"%ld,",record.dwTimeStamp);

		if (record.bip1 || record.bip2 || record.bip3 || record.bip4) {
			fprintf(out,"%d_%d_%d_%d-",
						record.bip1,record.bip2,
						record.bip3,record.bip4);
		}
		
		fprintf(out,"%ld-%d,",record.dwPID,record.wUser);
		fprintf(out,"%ld,",record.dwResponseTime);
		fprintf(out,"%d,",record.wResponseCode);
		fprintf(out,"%ld,",record.dwReplySize);
        
		// The full URL is taken from the urlFilename:
		LONG curoff = urlFile->Seek(0,CFile::current);
		UINT nbytes = urlFile->Read(buff,sizeof(buff));
		if (nbytes == 0) {
			fprintf(stderr," A problem occured: url filename smaller than data file ?\n");
			continue;
		}

		// URL are ended by a '\0': find it end seek the file pointer:
		int len = strlen(buff);	// there should always be a '\0' in the read buffer
		urlFile->Seek(curoff + len + 1,CFile::begin);

		fprintf(out,"%s\n",buff);
	}

    return 0;
}

/* EOF */